MockArquivos = [
    "ARQUIVO DE DADOS GERAIS     : nwlistcf.dat\n",
    "ARQUIVO DE CORTES DE BENDERS: cortes.dat\n",
    "ARQUIVO DE CABECALHO CORTES : cortesh.dat\n",
    "ARQUIVO P/DESPACHO HIDROTERM: newdesp.dat\n",
    "ARQUIVO DE ESTADOS CORTES   : cortese.dat\n",
    "ARQUIVO DE ENERGIAS FORWARD : energiaf.dat\n",
    "ARQUIVO DE RESTRICOES SAR   : rsar.dat\n",
    "ARQUIVO DE CABECALHO SAR    : rsarh.dat\n",
    "ARQUIVO DE INDICE SAR       : rsari.dat\n",
    "ARQUIVO LISTAGEM CORTES     : nwlistcf.rel\n",
    "ARQUIVO LISTAGEM ESTADOS FCF: estados.rel\n",
    "ARQUIVO LISTAGEM SAR        : rsar.rel\n",
    "ARQUIVO DE ENERGIAS X FORW  : energiaxf.dat\n",
    "ARQUIVO DE VAZAO FORWARD    : vazaof.dat\n",
    "ARQUIVO DE VAZAO X FORWARD  : vazaoxf.dat\n",
]
