MockBlocoVarreduraShist = [
    "VARREDURA    E   ANO INICIO VARREDURA\n",
    "XXX.XXX.\n",
    "  1 1932\n",
]

MockBlocoSeriesSimulacaoShist = [
    "SERIES PARA SIMULACAO\n",
    "XXXX\n",
    "9999\n",
]

MockShist = MockBlocoVarreduraShist + MockBlocoSeriesSimulacaoShist
