# Inclui os membros

# Deprecated
from .avl_cortesfpha_nwv import AvlCortesFpha  # noqa
from .avl_desvfpha_s import AvlDesvFphaS  # noqa
from .avl_desvfpha_v_q import AvlDesvFphaVQ  # noqa
from .eco_fpha import EcoFpha  # noqa
from .nwv_avl_evap import NwvAvlEvap  # noqa
from .nwv_cortes_evap import NwvCortesEvap  # noqa
from .nwv_eco_evap import NwvEcoEvap  # noqa

from .abertura import Abertura  # noqa
from .adterm import Adterm  # noqa
from .agrint import Agrint  # noqa
from .arquivos import Arquivos  # noqa
from .bid import BID  # noqa
from .cadic import Cadic  # noqa
from .caso import Caso  # noqa
from .clasgas import Clasgas  # noqa
from .clast import Clast  # noqa
from .confhd import Confhd  # noqa
from .conft import Conft  # noqa
from .curva import Curva  # noqa
from .cvar import Cvar  # noqa
from .dger import Dger  # noqa
from .dsvagua import Dsvagua  # noqa
from .eafpast import Eafpast  # noqa
from .elnino import ElNino  # noqa
from .ensoaux import ENSOAux  # noqa
from .exph import Exph  # noqa
from .expt import Expt  # noqa
from .gee import GEE  # noqa
from .ghmin import Ghmin  # noqa
from .gtminpat import Gtminpat  # noqa
from .hidr import Hidr  # noqa
from .itaipu import Itaipu  # noqa
from .manutt import Manutt  # noqa
from .modif import Modif  # noqa
from .newavetim import Newavetim  # noqa
from .parp import Parp  # noqa
from .parpvaz import Parpvaz  # noqa
from .parpeol import Parpeol  # noqa
from .patamar import Patamar  # noqa
from .penalid import Penalid  # noqa
from .perda import Perda  # noqa
from .pmo import Pmo  # noqa
from .re import Re  # noqa
from .ree import Ree  # noqa
from .sar import Sar  # noqa
from .shist import Shist  # noqa
from .selcor import Selcor  # noqa
from .sistema import Sistema  # noqa
from .tecno import Tecno  # noqa
from .term import Term  # noqa
from .vazoes import Vazoes  # noqa
from .vazpast import Vazpast  # noqa
from .volref_saz import VolrefSaz  # noqa
from .fpha_cortes import FphaCortes  # noqa
from .fpha_avl_desv_s import FphaAvlDesvS  # noqa
from .fpha_avl_desv_v_q import FphaAvlDesvVQ  # noqa
from .fpha_eco import FphaEco  # noqa
from .evap_avl_desv import EvapAvlDesv  # noqa
from .evap_cortes import EvapCortes  # noqa
from .evap_eco import EvapEco  # noqa
from .energiaf import Energiaf  # noqa
from .energiab import Energiab  # noqa
from .energias import Energias  # noqa
from .enavazf import Enavazf  # noqa
from .enavazb import Enavazb  # noqa
from .engnat import Engnat  # noqa
from .vazaof import Vazaof  # noqa
from .vazaob import Vazaob  # noqa
from .vazaos import Vazaos  # noqa
from .vazinat import Vazinat  # noqa
