MockSelcor = [
    "PARAMETROS UTILIZADO NA SELECAO DE CORTES DE BENDERS\n",
    "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX   XXXX  XXXX \n",
    "ITERACAO INICIAL APLICACAO DA SELECAO DE CORTES                0002\n",
    "TAMANHO DA JANELA DE CORTES ATIVOS (k2)                        0000\n",
    "QDE DE CORTES A SEREM ADICIONADOS POR ITERACAO (NADIC)         0012\n",
    "CONSIDERA CORTES DA PROPRIA ITERACAO                           0001\n",
    "IMPRIME RELATORIO DE SAIDA (selcor-xxxx.log)                   0000\n",
    "PERIODO INICIAL E FINAL PARA IMPRESSAO                         0007  0015    (campo 1 vazio ou zero: todo o horizonte)\n",
    "SERITE INICIAL E FINAL PARA IMPRESSAO                          0002  0009    (campo 1 vazio ou zero: todas as series da politica)\n",
]
