MockBlocoPeriodoImpressao = [
    " INI FIM FC (FC = 1: IMPRIME TODOS CORTES, FC = 0: IMPRIME APENAS CORTES VALIDOS NA ULTIMA ITERACAO)\n",
    " XXX XXX X    \n",
    "   3  60 0\n",
]

MockBlocoOpcoesImpressao = [
    " OPCOES DE IMPRESSAO : 01 - CORTES FCF  02 - ESTADOS FCF  03 - RESTRICAO SAR\n",
    " XX XX XX (SE 99 CONSIDERA TODAS)\n",
    " 99\n",
]

MockNwlistcfdat = MockBlocoPeriodoImpressao + MockBlocoOpcoesImpressao
